##

# Do some magic in order to make xmmsapi to see xmmsvalue
import sys, os
# XXX Do not prepend the directory. The threading import symbols from a 'collections'
# module that is not ours !!!
sys.path.append(os.path.dirname(__file__))

import xmmsapi, xmmsvalue
from xmmsapi import XMMS, XMMSResult, userconfdir_get
from xmmsvalue import XMMSValue
from xmmsvalue import coll_parse
from sync import XMMSSync, XMMSError
from propdict import PropDict
from consts import *

# Now, it is safe to remove the directory from the path.
sys.path.remove(os.path.dirname(__file__))
del sys, os



#Py3k compat
try:
	a = basestring
	del a
except NameError:
	basestring = str

class PropDict(dict):
    def __init__(self, srcs):
        dict.__init__(self)
        self._sources = srcs

    def set_source_preference(self, sources):
        """
        Change list of source preference
        This method has been deprecated and should no longer be used.
        """
        raise DeprecationWarning("This method has been deprecated and should no longer be used. Set the sources list using the 'sources' property.")
        self._set_sources(sources)

    def has_key(self, item):
        try:
            self.__getitem__(item)
            return True
        except KeyError:
            return False

    def __contains__(self, item):
        return self.has_key(item)

    def __getitem__(self, item):
        if isinstance(item, basestring):
            for src in self._sources:
                if src.endswith('*'):
                    for k in self:
                        if k[0].startswith(src[:-1]) and k[1] == item:
                            return dict.__getitem__(self, k)
                try:
                    t = dict.__getitem__(self, (src, item))
                    return t
                except KeyError:
                    pass
            raise KeyError(item)
        return dict.__getitem__(self, item)

    def get(self, item, default=None):
        try:
            return self[item]
        except KeyError:
            return default

    def _get_sources(self):
        return self._sources
    def _set_sources(self, val):
        if isinstance(val, basestring):
            raise TypeError("Need a sequence of sources")
        for i in val:
            if not isinstance(i, basestring):
                raise TypeError("Sources need to be strings")
        self._sources = val
    sources = property(_get_sources, _set_sources)





import xmmsapi

class XMMSSync:
    """
    A wrapper for the xmmsclient.XMMS class which simplifies synchronous
    communication with the XMMS2 daemon.

    Instances of this class may be used just like regular xmmsclient.XMMS
    objects, except that instead of returning an XMMSResult instance, the
    value associated with the result is returned.  If the XMMSResult
    indicates an error, an XMMSError is raised instead of returning the
    value.
    """
    def __init__(self, clientname=None, xmms=None):
        """
        This constructor takes two optional arguments. If xmms is omitted
        it will create a new underlying XMMS class otherwise it will use
        the one supplied. Clientname is the name of the client and will
        default to "Unnamed Python Client"
        """
        if xmms is None:
            self.__xmms = xmmsapi.XMMS(clientname)
        else:
            self.__xmms = xmms

    def __getattr__(self, name):
        attr = getattr(self.__xmms, name)
        if hasattr(attr, '__call__'):
            def _(*args, **kwargs):
                ret = attr(*args, **kwargs)
                if isinstance(ret, xmmsapi.XMMSResult):
                    ret.wait()
                    if ret.iserror():
                        raise XMMSError(ret.value())
                    return ret.value()
                return ret
            try:
                _.__doc__ = attr.__doc__
                _.func_name = '<sync version of %s>' % name
            except:
                pass
            return _
        else:
            return attr

    def __dir__(self):
        return dir(self.__xmms)

class XMMSError(Exception):
    """
    Thrown when a synchronous method call on an XMMS client object fails.
    """
    pass



# encoding: utf-8
"""

 A GLib connector for PyGTK - 
	to use with your cool PyGTK xmms2 client.
 Tobias Rundstrom <tru@xmms.org>
 Raphaël Bois <virtualdust@gmail.com>

 Just create the GLibConnector() class with a xmmsclient as argument

"""

from gobject import io_add_watch, IO_OUT, IO_IN, source_remove

class GLibConnector:
	def __init__(self, xmms):
		self.in_id = None
		self.out_id = None
		self.reconnect(xmms)

	def need_out(self, i):
		if self.xmms.want_ioout() and self.out_id is None:
			self.out_id = io_add_watch(self.xmms.get_fd(), IO_OUT, self.handle_out)

	def handle_in(self, source, cond):
		if cond == IO_IN:
			return self.xmms.ioin()

		return True

	def handle_out(self, source, cond):
		if cond == IO_OUT and self.xmms.want_ioout():
			self.xmms.ioout()
		if not self.xmms.want_ioout():
			self.out_id = None

		return not self.out_id is None

	def reconnect(self, xmms=None):
		self.disconnect()
		if not xmms is None:
			self.xmms = xmms
		self.xmms.set_need_out_fun(self.need_out)
		self.in_id = io_add_watch(self.xmms.get_fd(), IO_IN, self.handle_in)
		self.out_id = None

	def disconnect(self):
		if not self.in_id is None:
			source_remove(self.in_id)
			self.in_id = None
		if not self.out_id is None:
			source_remove(self.out_id)
			self.out_id = None
